@@ -21,7 +21,7 @@ using SHA
 export UUID, SHA1, VersionRange, VersionSpec,
     PackageSpec, PackageEntry, EnvCache, Context, GitRepo, Context!, Manifest, Project, err_rep,
     PkgError, pkgerror,
-    has_name, has_uuid, is_stdlib, is_or_was_stdlib, stdlib_version, is_unregistered_stdlib, stdlibs, write_env, write_env_usage, parse_toml,
+    has_name, has_uuid, is_stdlib, is_or_was_stdlib, stdlib_version, is_unregistered_stdlib, stdlibs, stdlib_infos, write_env, write_env_usage, parse_toml,
     project_resolve!, project_deps_resolve!, manifest_resolve!, registry_resolve!, stdlib_resolve!, handle_repos_develop!, handle_repos_add!, ensure_resolved,
     registered_name,
     manifest_info,
@@ -85,7 +85,8 @@ end
 
 Base.:(==)(r1::GitRepo, r2::GitRepo) =
     r1.source == r2.source && r1.rev == r2.rev && r1.subdir == r2.subdir
-
+Base.hash(r::GitRepo, h::UInt) =
+    foldr(hash, [r.source, r.rev, r.subdir], init=h)
 
 mutable struct PackageSpec
     name::Union{Nothing,String}
@@ -119,11 +120,15 @@ PackageSpec(name::AbstractString, uuid::UUID) = PackageSpec(;name=name, uuid=uui
 PackageSpec(name::AbstractString, version::VersionTypes) = PackageSpec(;name=name, version=version)::PackageSpec
 PackageSpec(n::AbstractString, u::UUID, v::VersionTypes) = PackageSpec(;name=n, uuid=u, version=v)::PackageSpec
 
+# XXX: These definitions are a bit fishy. It seems to be used in an `==` call in status printing
 function Base.:(==)(a::PackageSpec, b::PackageSpec)
     return a.name == b.name && a.uuid == b.uuid && a.version == b.version &&
     a.tree_hash == b.tree_hash && a.repo == b.repo && a.path == b.path &&
     a.pinned == b.pinned
 end
+function Base.hash(a::PackageSpec, h::UInt)
+    return foldr(hash, [a.name, a.uuid, a.version, a.tree_hash, a.repo, a.path, a.pinned], init=h)
+end
 
 function err_rep(pkg::PackageSpec)
     x = pkg.name !== nothing && pkg.uuid !== nothing ? x = "$(pkg.name) [$(string(pkg.uuid)[1:8])]" :
@@ -179,7 +184,7 @@ function projectfile_path(env_path::String; strict=false)
 end
 
 function manifestfile_path(env_path::String; strict=false)
-    for name in Base.manifest_names
+    for name in (Base.manifest_names..., "AppManifest.toml")
         maybe_file = joinpath(env_path, name)
         isfile(maybe_file) && return maybe_file
     end
@@ -233,6 +238,13 @@ end
 Base.:(==)(t1::Compat, t2::Compat) = t1.val == t2.val
 Base.hash(t::Compat, h::UInt) = hash(t.val, h)
 
+struct AppInfo
+    name::String
+    julia_command::Union{String, Nothing}
+    submodule::Union{String, Nothing}
+    julia_flags::Vector{String}
+    other::Dict{String, Any}
+end
 Base.@kwdef mutable struct Project
     other::Dict{String,Any} = Dict{String,Any}()
     # Fields
@@ -240,7 +252,7 @@ Base.@kwdef mutable struct Project
     uuid::Union{UUID, Nothing} = nothing
     version::Union{VersionTypes, Nothing} = nothing
     manifest::Union{String, Nothing} = nothing
-    path::Union{String, Nothing} = nothing
+    entryfile::Union{String, Nothing} = nothing
     # Sections
     deps::Dict{String,UUID} = Dict{String,UUID}()
     # deps that are also in weakdeps for backwards compat
@@ -251,27 +263,21 @@ Base.@kwdef mutable struct Project
     exts::Dict{String,Union{Vector{String}, String}} = Dict{String,String}()
     extras::Dict{String,UUID} = Dict{String,UUID}()
     targets::Dict{String,Vector{String}} = Dict{String,Vector{String}}()
+    apps::Dict{String, AppInfo} = Dict{String, AppInfo}()
     compat::Dict{String,Compat} = Dict{String,Compat}()
     sources::Dict{String,Dict{String, String}} = Dict{String,Dict{String, String}}()
+    workspace::Dict{String, Any} = Dict{String, Any}()
 end
 Base.:(==)(t1::Project, t2::Project) = all(x -> (getfield(t1, x) == getfield(t2, x))::Bool, fieldnames(Project))
 Base.hash(t::Project, h::UInt) = foldr(hash, [getfield(t, x) for x in fieldnames(Project)], init=h)
 
-# only hash the deps and compat fields as they are the only fields that affect a resolve
-function project_resolve_hash(t::Project)
-    iob = IOBuffer()
-    # Handle deps in both [deps] and [weakdeps]
-    _deps_weak = Dict(intersect(t.deps, t.weakdeps))
-    deps = filter(p->!haskey(_deps_weak, p.first), t.deps)
-    foreach(((name, uuid),) -> println(iob, name, "=", uuid), sort!(collect(deps); by=first))
-    foreach(((name, compat),) -> println(iob, name, "=", compat.val), sort!(collect(t.compat); by=first))
-    return bytes2hex(sha1(seekstart(iob)))
-end
+
 
 Base.@kwdef mutable struct PackageEntry
     name::Union{String,Nothing} = nothing
     version::Union{VersionNumber,Nothing} = nothing
     path::Union{String,Nothing} = nothing
+    entryfile::Union{String,Nothing} = nothing
     pinned::Bool = false
     repo::GitRepo = GitRepo()
     tree_hash::Union{Nothing,SHA1} = nothing
@@ -279,23 +285,27 @@ Base.@kwdef mutable struct PackageEntry
     weakdeps::Dict{String,UUID} = Dict{String,UUID}()
     exts::Dict{String,Union{Vector{String}, String}} = Dict{String,String}()
     uuid::Union{Nothing, UUID} = nothing
+    apps::Dict{String, AppInfo} = Dict{String, AppInfo}() # used by AppManifest.toml
     other::Union{Dict,Nothing} = nothing
 end
 Base.:(==)(t1::PackageEntry, t2::PackageEntry) = t1.name == t2.name &&
     t1.version == t2.version &&
     t1.path == t2.path &&
+    t1.entryfile == t2.entryfile &&
     t1.pinned == t2.pinned &&
     t1.repo == t2.repo &&
     t1.tree_hash == t2.tree_hash &&
     t1.deps == t2.deps &&
     t1.weakdeps == t2.weakdeps &&
     t1.exts == t2.exts &&
-    t1.uuid == t2.uuid
+    t1.uuid == t2.uuid &&
+    t1.apps == t2.apps
     # omits `other`
-Base.hash(x::PackageEntry, h::UInt) = foldr(hash, [x.name, x.version, x.path, x.pinned, x.repo, x.tree_hash, x.deps, x.weakdeps, x.exts, x.uuid], init=h)  # omits `other`
+Base.hash(x::PackageEntry, h::UInt) = foldr(hash, [x.name, x.version, x.path, x.entryfile, x.pinned, x.repo, x.tree_hash, x.deps, x.weakdeps, x.exts, x.uuid], init=h)  # omits `other`
 
 Base.@kwdef mutable struct Manifest
     julia_version::Union{Nothing,VersionNumber} = nothing # only set to VERSION when resolving
+    project_hash::Union{Nothing,SHA1} = nothing
     manifest_format::VersionNumber = v"2.0.0"
     deps::Dict{UUID,PackageEntry} = Dict{UUID,PackageEntry}()
     other::Dict{String,Any} = Dict{String,Any}()
@@ -330,6 +340,31 @@ function Base.show(io::IO, pkg::PackageEntry)
     print(io, ")")
 end
 
+function find_root_base_project(start_project::String)
+    project_file = start_project
+    while true
+        base_project_file = Base.base_project(project_file)
+        base_project_file === nothing && return project_file
+        project_file = base_project_file
+    end
+end
+
+function collect_workspace(base_project_file::String, d::Dict{String, Project}=Dict{String, Project}())
+    base_project = read_project(base_project_file)
+    d[base_project_file] = base_project
+    base_project_file_dir = dirname(base_project_file)
+
+    projects = get(base_project.workspace, "projects", nothing)::Union{Nothing,Vector{String}}
+    projects === nothing && return d
+    project_paths = [abspath(base_project_file_dir, project) for project in projects]
+    for project_path in project_paths
+        project_file = Base.locate_project_file(abspath(project_path))
+        if project_file isa String
+            collect_workspace(project_file, d)
+        end
+    end
+    return d
+end
 
 mutable struct EnvCache
     # environment info:
@@ -341,6 +376,7 @@ mutable struct EnvCache
     pkg::Union{PackageSpec, Nothing}
     # cache of metadata:
     project::Project
+    workspace::Dict{String,Project} # paths relative to base
     manifest::Manifest
     # What these where at creation of the EnvCache
     original_project::Project
@@ -348,7 +384,9 @@ mutable struct EnvCache
 end
 
 function EnvCache(env::Union{Nothing,String}=nothing)
+    # @show env
     project_file = find_project_file(env)
+    # @show project_file
     project_dir = dirname(project_file)
     # read project file
     project = read_project(project_file)
@@ -363,9 +401,19 @@ function EnvCache(env::Union{Nothing,String}=nothing)
     else
         project_package = nothing
     end
-    # determine manifest file
-    dir = abspath(project_dir)
+
     manifest_file = project.manifest
+    root_base_proj_file = find_root_base_project(project_file)
+    workspace = Dict{String, Project}()
+    if isfile(root_base_proj_file)
+        if root_base_proj_file !== project_file
+            manifest_file = manifestfile_path(dirname(root_base_proj_file))
+        end
+        workspace = collect_workspace(root_base_proj_file)
+        delete!(workspace, abspath(project_file))
+    end
+
+    dir = abspath(project_dir)
     manifest_file = manifest_file !== nothing ?
         (isabspath(manifest_file) ? manifest_file : abspath(dir, manifest_file)) :
         manifestfile_path(dir)::String
@@ -377,6 +425,7 @@ function EnvCache(env::Union{Nothing,String}=nothing)
         manifest_file,
         project_package,
         project,
+        workspace,
         manifest,
         deepcopy(project),
         deepcopy(manifest),
@@ -385,6 +434,22 @@ function EnvCache(env::Union{Nothing,String}=nothing)
     return env′
 end
 
+# Convert a path from project-relative to manifest-relative
+# If path is absolute, returns it as-is
+function project_path_to_manifest_path(project_file::String, manifest_file::String, path::String)
+    isabspath(path) && return path
+    abs_path = Pkg.safe_realpath(joinpath(dirname(project_file), path))
+    return relpath(abs_path, Pkg.safe_realpath(dirname(manifest_file)))
+end
+
+# Convert a path from manifest-relative to project-relative
+# If path is absolute, returns it as-is
+function manifest_path_to_project_path(project_file::String, manifest_file::String, path::String)
+    isabspath(path) && return path
+    abs_path = Pkg.safe_realpath(joinpath(dirname(manifest_file), path))
+    return relpath(abs_path, Pkg.safe_realpath(dirname(project_file)))
+end
+
 include("project.jl")
 include("manifest.jl")
 
@@ -412,7 +477,8 @@ Base.@kwdef mutable struct Context
     julia_version::Union{VersionNumber,Nothing} = VERSION
 end
 
-project_uuid(env::EnvCache) = env.pkg === nothing ? Base.dummy_uuid(env.project_file) : env.pkg.uuid
+project_uuid(env::EnvCache) = project_uuid(env.project, env.project_file)
+project_uuid(project::Project, project_file::String) = @something(project.uuid, Base.dummy_uuid(project_file))
 collides_with_project(env::EnvCache, pkg::PackageSpec) =
     is_project_name(env, pkg.name) || is_project_uuid(env, pkg.uuid)
 is_project(env::EnvCache, pkg::PackageSpec) = is_project_uuid(env, pkg.uuid)
@@ -425,8 +491,8 @@ is_project_uuid(env::EnvCache, uuid::UUID) = project_uuid(env) == uuid
 # Context #
 ###########
 
-const FORMER_STDLIBS = ["DelimitedFiles", "Statistics"]
-const FORMER_STDLIBS_UUIDS = Set{UUID}()
+const UPGRADABLE_STDLIBS = ["DelimitedFiles", "Statistics"]
+const UPGRADABLE_STDLIBS_UUIDS = Set{UUID}()
 const STDLIB = Ref{Union{DictStdLibs, Nothing}}(nothing)
 function load_stdlib()
     stdlib = DictStdLibs()
@@ -438,51 +504,77 @@ function load_stdlib()
         v_str = get(project, "version", nothing)::Union{String, Nothing}
         version = isnothing(v_str) ? nothing : VersionNumber(v_str)
         nothing === uuid && continue
-        if name in FORMER_STDLIBS
-            push!(FORMER_STDLIBS_UUIDS, UUID(uuid))
+        if name in UPGRADABLE_STDLIBS
+            push!(UPGRADABLE_STDLIBS_UUIDS, UUID(uuid))
             continue
         end
-        stdlib[UUID(uuid)] = (name, version)
+        deps = UUID.(values(get(project, "deps", Dict{String,Any}())))
+        weakdeps = UUID.(values(get(project, "weakdeps", Dict{String,Any}())))
+        stdlib[UUID(uuid)] = StdlibInfo(name, Base.UUID(uuid), version, deps, weakdeps)
     end
     return stdlib
 end
 
 function stdlibs()
+    # This maintains a compatible format for `stdlibs()`, but new code should always
+    # prefer `stdlib_infos` as it is more future-proofed, by returning a structure
+    # rather than a tuple of elements.
+    return Dict(uuid => (info.name, info.version) for (uuid, info) in stdlib_infos())
+end
+function stdlib_infos()
     if STDLIB[] === nothing
         STDLIB[] = load_stdlib()
     end
     return STDLIB[]
 end
-is_stdlib(uuid::UUID) = uuid in keys(stdlibs())
+is_stdlib(uuid::UUID) = uuid in keys(stdlib_infos())
 # Includes former stdlibs
 function is_or_was_stdlib(uuid::UUID, julia_version::Union{VersionNumber, Nothing})
-    return is_stdlib(uuid, julia_version) || uuid in FORMER_STDLIBS_UUIDS
+    return is_stdlib(uuid, julia_version) || uuid in UPGRADABLE_STDLIBS_UUIDS
 end
 
 
+function historical_stdlibs_check()
+    if isempty(STDLIBS_BY_VERSION)
+        pkgerror("If you want to set `julia_version`, you must first populate the `STDLIBS_BY_VERSION` global constant.  Try `using HistoricalStdlibVersions`")
+    end
+end
+
 # Find the entry in `STDLIBS_BY_VERSION`
 # that corresponds to the requested version, and use that.
 # If we can't find one, defaults to `UNREGISTERED_STDLIBS`
 function get_last_stdlibs(julia_version::VersionNumber; use_historical_for_current_version = false)
     if !use_historical_for_current_version && julia_version == VERSION
-        return stdlibs()
+        return stdlib_infos()
     end
+    historical_stdlibs_check()
     last_stdlibs = UNREGISTERED_STDLIBS
-    if isempty(STDLIBS_BY_VERSION)
-        pkgerror("If you want to set `julia_version`, you must first populate the `STDLIBS_BY_VERSION` global constant")
-    end
+    last_version = nothing
+
     for (version, stdlibs) in STDLIBS_BY_VERSION
+        if !isnothing(last_version) && last_version > version
+            pkgerror("STDLIBS_BY_VERSION must be sorted by version number")
+        end
         if VersionNumber(julia_version.major, julia_version.minor, julia_version.patch) < version
             break
         end
         last_stdlibs = stdlibs
+        last_version = version
     end
+    # Serving different patches is safe-ish, but different majors or minors is most likely not.
+    if last_version !== nothing && (last_version.major != julia_version.major || last_version.minor != julia_version.minor)
+        pkgerror("Could not find a julia version in STDLIBS_BY_VERSION that matches the major & minor version of requested julia_version v$(julia_version)")
+    end
+
     return last_stdlibs
 end
 # If `julia_version` is set to `nothing`, that means (essentially) treat all registered
 # stdlibs as normal packages so that we get the latest versions of everything, ignoring
 # julia compat.  So we set the list of stdlibs to that of only the unregistered stdlibs.
-get_last_stdlibs(::Nothing) = UNREGISTERED_STDLIBS
+function get_last_stdlibs(::Nothing)
+    historical_stdlibs_check()
+    return UNREGISTERED_STDLIBS
+end
 
 # Allow asking if something is an stdlib for a particular version of Julia
 function is_stdlib(uuid::UUID, julia_version::Union{VersionNumber, Nothing})
@@ -506,10 +598,13 @@ function stdlib_version(uuid::UUID, julia_version::Union{VersionNumber,Nothing})
     if !(uuid in keys(last_stdlibs))
         return nothing
     end
-    return last_stdlibs[uuid][2]
+    return last_stdlibs[uuid].version
 end
 
-is_unregistered_stdlib(uuid::UUID) = haskey(UNREGISTERED_STDLIBS, uuid)
+function is_unregistered_stdlib(uuid::UUID)
+    historical_stdlibs_check()
+    return haskey(UNREGISTERED_STDLIBS, uuid)
+end
 
 Context!(kw_context::Vector{Pair{Symbol,Any}})::Context =
     Context!(Context(); kw_context...)
@@ -517,12 +612,60 @@ function Context!(ctx::Context; kwargs...)
     for (k, v) in kwargs
         setfield!(ctx, k, v)
     end
+
+    # Highlight for logging purposes if julia_version is set to a different version than current VERSION
+    if haskey(kwargs, :julia_version) && ctx.julia_version !== nothing && ctx.julia_version != VERSION
+        Pkg.printpkgstyle(
+            ctx.io, :Context,
+            "Pkg is operating with julia_version set to `$(ctx.julia_version)`",
+            color = Base.warn_color()
+        )
+    end
+
     return ctx
 end
 
-function write_env_usage(source_file::AbstractString, usage_filepath::AbstractString)
+function load_workspace_weak_deps(env::EnvCache)
+    # TODO: Possible name collision between projects in a workspace?
+    weakdeps = Dict{String, UUID}()
+    merge!(weakdeps, env.project.weakdeps)
+    for (_, proj) in env.workspace
+        merge!(weakdeps, proj.weakdeps)
+    end
+    return weakdeps
+end
+
+# only hash the deps and compat fields as they are the only fields that affect a resolve
+function workspace_resolve_hash(env::EnvCache)
+    # Handle deps in both [deps] and [weakdeps]
+    deps = Dict(pkg.name => pkg.uuid for pkg in Pkg.Operations.load_direct_deps(env))
+    weakdeps = load_workspace_weak_deps(env)
+    alldeps = merge(deps, weakdeps)
+    compats = Dict(name => Pkg.Operations.get_compat_workspace(env, name) for (name, uuid) in alldeps)
+    iob = IOBuffer()
+    for (name, uuid) in sort!(collect(deps); by=first)
+        println(iob, name, "=", uuid)
+    end
+    println(iob)
+    for (name, uuid) in sort!(collect(weakdeps); by=first)
+        println(iob, name, "=", uuid)
+    end
+    println(iob)
+    for (name, compat) in sort!(collect(compats); by=first)
+        println(iob, name, "=", compat)
+    end
+    str = String(take!(iob))
+    return bytes2hex(sha1(str))
+end
+
+
+write_env_usage(source_file::AbstractString, usage_filepath::AbstractString) =
+    write_env_usage([source_file], usage_filepath)
+
+function write_env_usage(source_files, usage_filepath::AbstractString)
     # Don't record ghost usage
-    !isfile(source_file) && return
+    source_files = filter(isfile, source_files)
+    isempty(source_files) && return
 
     # Ensure that log dir exists
     !ispath(logdir()) && mkpath(logdir())
@@ -544,7 +687,9 @@ function write_env_usage(source_file::AbstractString, usage_filepath::AbstractSt
         end
 
         # record new usage
-        usage[source_file] = [Dict("time" => timestamp)]
+        for source_file in source_files
+            usage[source_file] = [Dict("time" => timestamp)]
+        end
 
         # keep only latest usage info
         for k in keys(usage)
@@ -583,14 +728,16 @@ function read_package(path::String)
         pkgerror("expected a `uuid` entry in project file at `$(abspath(path))`")
     end
     name = project.name
-    pkgpath = joinpath(dirname(path), something(project.path, ""))
-    if !isfile(joinpath(pkgpath, "src", "$name.jl"))
-        pkgerror("expected the file `src/$name.jl` to exist for package `$name` at `$(dirname(path))`")
+    entry_point = something(project.entryfile, joinpath("src", "$(name).jl"))
+    pkgpath = joinpath(dirname(path), entry_point)
+    if !isfile(pkgpath)
+        pkgerror("expected the file `$pkgpath` to exist for package `$name` at `$(dirname(path))`")
     end
     return project
 end
 
-const refspecs = ["+refs/*:refs/remotes/cache/*"]
+const refspecs = ["+refs/heads/*:refs/remotes/cache/heads/*"]
+const refspecs_fallback = ["+refs/*:refs/remotes/cache/*"]
 
 function relative_project_path(project_file::String, path::String)
     # compute path relative the project
@@ -696,7 +843,7 @@ function handle_repo_develop!(ctx::Context, pkg::PackageSpec, shared::Bool)
         new = true
     end
     if !has_uuid(pkg)
-        resolve_projectfile!(pkg, dev_path)
+        resolve_projectfile!(pkg, joinpath(dev_path, pkg.repo.subdir === nothing ? "" : pkg.repo.subdir))
     end
     error_if_in_sysimage(pkg)
     pkg.path = shared ? dev_path : relative_project_path(ctx.env.manifest_file, dev_path)
@@ -795,15 +942,20 @@ function handle_repo_add!(ctx::Context, pkg::PackageSpec)
             if pkg.repo.rev === nothing
                 pkg.repo.rev = LibGit2.isattached(repo) ? LibGit2.branch(repo) : string(LibGit2.GitHash(LibGit2.head(repo)))
             end
-
-            obj_branch = get_object_or_branch(repo, pkg.repo.rev)
+            rev_or_hash = pkg.tree_hash === nothing ? pkg.repo.rev : pkg.tree_hash
+            obj_branch = get_object_or_branch(repo, rev_or_hash)
             fetched = false
             if obj_branch === nothing
                 fetched = true
                 GitTools.fetch(ctx.io, repo, repo_source_typed; refspecs=refspecs)
-                obj_branch = get_object_or_branch(repo, pkg.repo.rev)
+                obj_branch = get_object_or_branch(repo, rev_or_hash)
+                # If still not found, try with broader refspec as fallback
                 if obj_branch === nothing
-                    pkgerror("Did not find rev $(pkg.repo.rev) in repository")
+                    GitTools.fetch(ctx.io, repo, repo_source_typed; refspecs = refspecs_fallback)
+                    obj_branch = get_object_or_branch(repo, rev_or_hash)
+                end
+                if obj_branch === nothing
+                    pkgerror("Did not find rev $(rev_or_hash) in repository")
                 end
             end
             gitobject, isbranch = obj_branch
@@ -813,7 +965,7 @@ function handle_repo_add!(ctx::Context, pkg::PackageSpec)
             ispinned = innerentry !== nothing && innerentry.pinned
             if isbranch && !fetched && !ispinned
                 GitTools.fetch(ctx.io, repo, repo_source_typed; refspecs=refspecs)
-                gitobject, isbranch = get_object_or_branch(repo, pkg.repo.rev)
+                gitobject, isbranch = get_object_or_branch(repo, rev_or_hash)
             end
 
             # Now we have the gitobject for our ref, time to find the tree hash for it
@@ -923,7 +1075,10 @@ end
 
 # Disambiguate name/uuid package specifications using project info.
 function project_deps_resolve!(env::EnvCache, pkgs::AbstractVector{PackageSpec})
-    uuids = env.project.deps
+    uuids = copy(env.project.deps)
+    for (_, project) in env.workspace
+        merge!(uuids, project.deps)
+    end
     names = Dict(uuid => name for (name, uuid) in uuids)
     for pkg in pkgs
         if has_name(pkg) && !has_uuid(pkg) && pkg.name in keys(uuids)
@@ -974,13 +1129,17 @@ function stdlib_resolve!(pkgs::AbstractVector{PackageSpec})
     for pkg in pkgs
         @assert has_name(pkg) || has_uuid(pkg)
         if has_name(pkg) && !has_uuid(pkg)
-            for (uuid, (name, version)) in stdlibs()
-                name == pkg.name && (pkg.uuid = uuid)
+            for (uuid, info) in stdlib_infos()
+                if info.name == pkg.name
+                    pkg.uuid = uuid
+                end
             end
         end
         if !has_name(pkg) && has_uuid(pkg)
-            name, version = get(stdlibs(), pkg.uuid, (nothing, nothing))
-            nothing !== name && (pkg.name = name)
+            info = get(stdlib_infos(), pkg.uuid, nothing)
+            if info !== nothing
+                pkg.name = info.name
+            end
         end
     end
 end
@@ -1106,13 +1265,12 @@ function write_env(env::EnvCache; update_undo=true,
                    skip_writing_project::Bool=false)
     # Verify that the generated manifest is consistent with `sources`
     for (pkg, uuid) in env.project.deps
-        path, repo = get_path_repo(env.project, pkg)
+        path, repo = get_path_repo(env.project, env.project_file, env.manifest_file, pkg)
         entry = manifest_info(env.manifest, uuid)
         if path !== nothing
-            @assert entry.path == path
+            @assert normpath(entry.path) == normpath(path)
         end
         if repo != GitRepo()
-            @assert entry.repo.source == repo.source
             if repo.rev !== nothing
                 @assert entry.repo.rev == repo.rev
             end
@@ -1120,6 +1278,19 @@ function write_env(env::EnvCache; update_undo=true,
                 @assert entry.repo.subdir == repo.subdir
             end
         end
+        if entry !== nothing
+            if entry.path !== nothing
+                # Convert path from manifest-relative to project-relative before writing
+                project_relative_path = manifest_path_to_project_path(env.project_file, env.manifest_file, entry.path)
+                env.project.sources[pkg] = Dict("path" => project_relative_path)
+            elseif entry.repo != GitRepo()
+                d = Dict{String, String}()
+                entry.repo.source !== nothing && (d["url"] = entry.repo.source)
+                entry.repo.rev !== nothing && (d["rev"] = entry.repo.rev)
+                entry.repo.subdir !== nothing && (d["subdir"] = entry.repo.subdir)
+                env.project.sources[pkg] = d
+            end
+        end
     end
     if (env.project != env.original_project) && (!skip_writing_project)
         write_project(env)
