@@ -8,7 +8,7 @@ using Pkg.Types
 using Pkg.Types: VersionBound
 using UUIDs
 using Pkg.Resolve
-import Pkg.Resolve: VersionWeight, add_reqs!, simplify_graph!, ResolverError, Fixed, Requires
+import Pkg.Resolve: VersionWeight, add_reqs!, simplify_graph!, ResolverError, ResolverTimeoutError, Fixed, Requires
 
 include("utils.jl")
 using .Utils
@@ -161,13 +161,18 @@ end
     want_data = Dict("B"=>v"1")
     @test resolve_tst(deps_data, reqs_data, want_data)
 
+    # require A (must give an error)
+    reqs_data = Any[
+        ["A", "*"]
+    ]
+    @test_throws ResolverError resolve_tst(deps_data, reqs_data)
+
 
     VERBOSE && @info("SCHEME 5")
     ## DEPENDENCY SCHEME 5: THREE PACKAGES, DAG, WITH IMPLICIT INCONSISTENCY
     deps_data = Any[
         ["A", v"1", "B", "2-*"],
         ["A", v"1", "C", "2-*"],
-        # ["A", v"1", "julia", "10"],
         ["A", v"2", "B", "1"],
         ["A", v"2", "C", "1"],
         ["B", v"1", "C", "2-*"],
@@ -426,98 +431,321 @@ end
     want_data = Dict("A"=>v"1", "B"=>v"1", "C"=>v"1", "D"=>v"1")
     @test resolve_tst(deps_data, reqs_data, want_data)
 
+
+    VERBOSE && @info("SCHEME 12")
+    ## DEPENDENCY SCHEME 12: TWO PACKAGES, DAG, WEAK DEPENDENCY
+    deps_data = Any[
+        ["A", v"1", "B", "1-*", :weak],
+        ["A", v"2", "B", "2-*", :weak],
+        ["B", v"1"],
+        ["B", v"2"]
+    ]
+
+    @test sanity_tst(deps_data)
+    @test sanity_tst(deps_data, pkgs=["A", "B"])
+    @test sanity_tst(deps_data, pkgs=["B"])
+    @test sanity_tst(deps_data, pkgs=["A"])
+
+    # require just B
+    reqs_data = Any[
+        ["B", "*"]
+    ]
+    want_data = Dict("B"=>v"2")
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require just A
+    reqs_data = Any[
+        ["A", "*"]
+    ]
+    want_data = Dict("A"=>v"2")
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require A and B
+    reqs_data = Any[
+        ["A", "*"],
+        ["B", "*"]
+    ]
+    want_data = Dict("A"=>v"2", "B"=>v"2")
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require A and B, invompatible versions
+    reqs_data = Any[
+        ["A", "2-*"],
+        ["B", "1"]
+    ]
+    @test_throws ResolverError resolve_tst(deps_data, reqs_data, want_data)
+
+
+    VERBOSE && @info("SCHEME 13")
+    ## DEPENDENCY SCHEME 13: LIKE 9 (SIX PACKAGES, DAG), WITH SOME WEAK DEPENDENCIES
+    deps_data = Any[
+        ["A", v"1"],
+        ["A", v"2"],
+        ["A", v"3"],
+        ["B", v"1", "A", "1"],
+        ["B", v"2", "A", "*"],
+        ["C", v"1", "A", "2", :weak],
+        ["C", v"2", "A", "2-*"],
+        ["D", v"1", "B", "1-*"],
+        ["D", v"2", "B", "2-*", :weak],
+        ["E", v"1", "D", "*"],
+        ["F", v"1", "A", "1-2"],
+        ["F", v"1", "E", "*"],
+        ["F", v"2", "C", "2-*"],
+        ["F", v"2", "E", "*"],
+    ]
+
+    @test sanity_tst(deps_data)
+
+    # require just F
+    reqs_data = Any[
+        ["F", "*"]
+    ]
+    want_data = Dict("A"=>v"3", "C"=>v"2",
+                    "D"=>v"2", "E"=>v"1", "F"=>v"2")
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require just F, lower version
+    reqs_data = Any[
+        ["F", "1"]
+    ]
+    want_data = Dict("A"=>v"2", "D"=>v"2",
+                    "E"=>v"1", "F"=>v"1")
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require F and B; force lower B version -> must bring down F, A, and D versions too
+    reqs_data = Any[
+        ["F", "*"],
+        ["B", "1"]
+    ]
+    want_data = Dict("A"=>v"1", "B"=>v"1", "D"=>v"1",
+                    "E"=>v"1", "F"=>v"1")
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require F and D; force lower D version -> must not bring down F version, and bring in B
+    reqs_data = Any[
+        ["F", "*"],
+        ["D", "1"]
+    ]
+    want_data = Dict("A"=>v"3", "B"=>v"2", "C"=>v"2",
+                    "D"=>v"1", "E"=>v"1", "F"=>v"2")
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require F and C; force lower C version -> must bring down F and A versions
+    reqs_data = Any[
+        ["F", "*"],
+        ["C", "1"]
+    ]
+    want_data = Dict("A"=>v"2", "C"=>v"1",
+                    "D"=>v"2", "E"=>v"1", "F"=>v"1")
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+
+    VERBOSE && @info("SCHEME 14")
+    ## DEPENDENCY SCHEME 14: A NASTY GRAPH WITH A LOCAL OPTIMUM
+    ## (REDUCED VERSION OF REALISTIC SCHEME 17 BELOW, ref Pkg.jl issue #3232)
+    deps_data = Any[
+        ["A", v"1", "X", "*"],
+        ["B", v"1"],
+        ["B", v"2"],
+        ["C", v"1"],
+        ["C", v"2", "G", "2"],
+        ["C", v"2", "H", "*"],
+        ["D", v"1"],
+        ["D", v"2", "C", "*"],
+        ["Y", v"0.1"],
+        ["Y", v"0.2.1", "B", "1"],
+        ["Y", v"0.2.2"],
+        ["X", v"0.1", "Y", "0.1"],
+        ["X", v"0.2", "Y", "0.2"],
+        ["E", v"1", "X", "0.1"],
+        ["F", v"1", "I", "1"],
+        ["G", v"1", "E", "*"],
+        ["G", v"2"],
+        ["H", v"1", "B", "*"],
+        ["H", v"1", "F", "*"],
+        ["H", v"1", "I", "*"],
+        ["I", v"1"],
+        ["I", v"2"],
+    ]
+
+    @test sanity_tst(deps_data)
+
+    # require A and D
+    reqs_data = Any[
+        ["A", "*"],
+        ["D", "*"],
+    ]
+    want_data = Dict(
+        "A"=>v"1",
+        "B"=>v"2",
+        "C"=>v"2",
+        "D"=>v"2",
+        "Y"=>v"0.2.2",
+        "X"=>v"0.2",
+        "F"=>v"1",
+        "G"=>v"2",
+        "H"=>v"1",
+        "I"=>v"1",
+    )
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require just D
+    reqs_data = Any[
+        ["D", "*"],
+    ]
+    want_data = Dict(
+        "B"=>v"2",
+        "C"=>v"2",
+        "D"=>v"2",
+        "F"=>v"1",
+        "G"=>v"2",
+        "H"=>v"1",
+        "I"=>v"1",
+    )
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require just A
+    reqs_data = Any[
+        ["A", "*"],
+    ]
+    want_data = Dict(
+        "A"=>v"1",
+        "Y"=>v"0.2.2",
+        "X"=>v"0.2",
+    )
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+    # require A, D, and lower version of Y
+    reqs_data = Any[
+        ["A", "*"],
+        ["D", "*"],
+        ["Y", "0.2.1"]
+    ]
+    want_data = Dict(
+        "A"=>v"1",
+        "B"=>v"1",
+        "C"=>v"2",
+        "D"=>v"2",
+        "Y"=>v"0.2.1",
+        "X"=>v"0.2",
+        "F"=>v"1",
+        "G"=>v"2",
+        "H"=>v"1",
+        "I"=>v"1",
+    )
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
+
+    VERBOSE && @info("SCHEME 15")
+    ## DEPENDENCY SCHEME 15: A GRAPH WITH A WEAK DEPENDENCE
+    ## (REDUCED VERSION OF A REALISTIC SCHEME, ref Pkg.jl issue #4030)
+    deps_data = Any[
+        ["A", v"1"],
+        ["A", v"2", "C", "*"],
+        ["B", v"1", "D", "1", :weak],
+        ["C", v"1", "E", "*"],
+        ["C", v"2", "E", "*"],
+        ["C", v"2", "B", "1"],
+        ["E", v"1", "D", "1"],
+        ["E", v"2", "F", "1"],
+        ["F", v"1", "D", "*"],
+        ["D", v"1"],
+        ["D", v"2"],
+    ]
+
+    @test sanity_tst(deps_data)
+
+    reqs_data = Any[
+        ["A", "*"],
+    ]
+    want_data = Dict(
+        "A" => v"2",
+        "B" => v"1",
+        "C" => v"2",
+        "D" => v"1",
+        "E" => v"2",
+        "F" => v"1",
+    )
+    @test resolve_tst(deps_data, reqs_data, want_data)
+
 end
 
 @testset "realistic" begin
-    VERBOSE && @info("SCHEME REALISTIC")
-    ## DEPENDENCY SCHEME 12: A REALISTIC EXAMPLE
+    tmp = mktempdir()
+    Pkg.PlatformEngines.unpack(joinpath(@__DIR__, "resolvedata.tar.gz"), tmp; verbose=false)
+
+    VERBOSE && @info("SCHEME REALISTIC 1")
+    ## DEPENDENCY SCHEME 15: A REALISTIC EXAMPLE
     ## ref Julia issue #21485
 
-    include("resolvedata1.jl")
+    include(joinpath(tmp, "resolvedata1.jl"))
 
     @test sanity_tst(ResolveData.deps_data, ResolveData.problematic_data)
     @test resolve_tst(ResolveData.deps_data, ResolveData.reqs_data, ResolveData.want_data)
 
-    ## DEPENDENCY SCHEME 13: A LARGER, MORE DIFFICULT REALISTIC EXAMPLE
+    VERBOSE && @info("SCHEME REALISTIC 2")
+    ## DEPENDENCY SCHEME 16: A LARGER, MORE DIFFICULT REALISTIC EXAMPLE
     ## ref Pkg.jl issue #1949
 
-    include("resolvedata2.jl")
+    include(joinpath(tmp, "resolvedata2.jl"))
 
     @test sanity_tst(ResolveData2.deps_data, ResolveData2.problematic_data)
     @test resolve_tst(ResolveData2.deps_data, ResolveData2.reqs_data, ResolveData2.want_data)
+
+    VERBOSE && @info("SCHEME REALISTIC 3")
+    ## DEPENDENCY SCHEME 17: AN EVEN LARGER, MORE DIFFICULT REALISTIC EXAMPLE
+    ## ref Pkg.jl issue #3232
+
+    include(joinpath(tmp, "resolvedata3.jl"))
+
+    @test sanity_tst(ResolveData3.deps_data, ResolveData3.problematic_data)
+    @test resolve_tst(ResolveData3.deps_data, ResolveData3.reqs_data, ResolveData3.want_data)
+
+    VERBOSE && @info("SCHEME REALISTIC 4")
+    ## DEPENDENCY SCHEME 18: AN UNSAT REALISTIC EXAMPLE IN WHICH THE RESOLVER HANGS FOR MINUTES
+    ## The resolve_tst call is intended to fail (by timeout) with validate_versions=false
+    ## ref Pkg.jl issue #3878
+
+    include(joinpath(tmp, "resolvedata4.jl"))
+
+    @test sanity_tst(ResolveData4.deps_data, ResolveData4.problematic_data)
+    withenv("JULIA_PKG_RESOLVE_MAX_TIME"=>10) do
+        @test_throws ResolverError resolve_tst(ResolveData4.deps_data, ResolveData4.reqs_data, ResolveData4.want_data)
+    end
+    withenv("JULIA_PKG_RESOLVE_MAX_TIME"=>1e-5) do
+        # this test may fail if graph preprocessing or the greedy solver get better
+        @test_throws ResolverTimeoutError resolve_tst(ResolveData4.deps_data, ResolveData4.reqs_data, ResolveData4.want_data; validate_versions=false)
+    end
+
 end
 
 @testset "nasty" begin
     VERBOSE && @info("SCHEME NASTY")
-    ## DEPENDENCY SCHEME 13: A NASTY CASE
+    ## DEPENDENCY SCHEME 19: A NASTY CASE
 
     include("NastyGenerator.jl")
-    deps_data, reqs_data, want_data, problematic_data = NastyGenerator.generate_nasty(5, 20, q=20, d=4, sat = true)
+    deps_data, reqs_data, want_data, problematic_data = NastyGenerator.generate_nasty(5, 20, q=20, d=4, sat=true)
 
     @test sanity_tst(deps_data, problematic_data)
     @test resolve_tst(deps_data, reqs_data, want_data)
 
-    deps_data, reqs_data, want_data, problematic_data = NastyGenerator.generate_nasty(5, 20, q=20, d=4, sat = false)
+    deps_data, reqs_data, want_data, problematic_data = NastyGenerator.generate_nasty(5, 20, q=20, d=4, sat=false)
 
     @test sanity_tst(deps_data, problematic_data)
     @test_throws ResolverError resolve_tst(deps_data, reqs_data)
 end
 
-@testset "Resolving for another version of Julia" begin
-    temp_pkg_dir() do dir
-        function find_by_name(versions, name)
-            idx = findfirst(p -> p.name == name, versions)
-            if idx === nothing
-                return nothing
-            end
-            return versions[idx]
-        end
-
-        # First, we're going to resolve for specific versions of Julia, ensuring we get the right dep versions:
-        Pkg.Registry.download_default_registries(Pkg.stdout_f())
-        ctx = Pkg.Types.Context(;julia_version=v"1.5")
-        versions, deps = Pkg.Operations._resolve(ctx.io, ctx.env, ctx.registries, [
-            Pkg.Types.PackageSpec(name="MPFR_jll", uuid=Base.UUID("3a97d323-0669-5f0c-9066-3539efd106a3")),
-        ], Pkg.Types.PRESERVE_TIERED, ctx.julia_version)
-        gmp = find_by_name(versions, "GMP_jll")
-        @test gmp !== nothing
-        @test gmp.version.major == 6 && gmp.version.minor == 1
-        ctx = Pkg.Types.Context(;julia_version=v"1.6")
-        versions, deps = Pkg.Operations._resolve(ctx.io, ctx.env, ctx.registries, [
-            Pkg.Types.PackageSpec(name="MPFR_jll", uuid=Base.UUID("3a97d323-0669-5f0c-9066-3539efd106a3")),
-        ], Pkg.Types.PRESERVE_TIERED, ctx.julia_version)
-        gmp = find_by_name(versions, "GMP_jll")
-        @test gmp !== nothing
-        @test gmp.version.major == 6 && gmp.version.minor == 2
-
-        # We'll also test resolving an "impossible" manifest; one that requires two package versions that
-        # are not both loadable by the same Julia:
-        ctx = Pkg.Types.Context(;julia_version=nothing)
-        versions, deps = Pkg.Operations._resolve(ctx.io, ctx.env, ctx.registries, [
-            # This version of GMP only works on Julia v1.6
-            Pkg.Types.PackageSpec(name="GMP_jll", uuid=Base.UUID("781609d7-10c4-51f6-84f2-b8444358ff6d"), version=v"6.2.0"),
-            # This version of MPFR only works on Julia v1.5
-            Pkg.Types.PackageSpec(name="MPFR_jll", uuid=Base.UUID("3a97d323-0669-5f0c-9066-3539efd106a3"), version=v"4.0.2"),
-        ], Pkg.Types.PRESERVE_TIERED, ctx.julia_version)
-        gmp = find_by_name(versions, "GMP_jll")
-        @test gmp !== nothing
-        @test gmp.version.major == 6 && gmp.version.minor == 2
-        mpfr = find_by_name(versions, "MPFR_jll")
-        @test mpfr !== nothing
-        @test mpfr.version.major == 4 && mpfr.version.minor == 0
-    end
-end
-
 @testset "Stdlib resolve smoketest" begin
     # All stdlibs should be installable and resolvable
     temp_pkg_dir() do dir
         Pkg.activate(temp=true)
-        Pkg.add(map(first, values(Pkg.Types.load_stdlib())))    # add all stdlibs
+        Pkg.add(map(x -> x.name, values(Pkg.Types.load_stdlib())))    # add all stdlibs
         iob = IOBuffer()
         Pkg.resolve(io = iob)
         str = String(take!(iob))
-        @test occursin(r"No Changes to .*Project.toml", str)
-        @test occursin(r"No Changes to .*Manifest.toml", str)
+        @test occursin(r"No packages added to or removed from .*Project.toml", str)
+        @test occursin(r"No packages added to or removed from .*Manifest.toml", str)
     end
 end
 
